@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2010-2014 Codership Oy <info@codership.com>
+// Copyright (C) 2010-2017 Codership Oy <info@codership.com>
 //
 
 #include "galera_common.hpp"
@@ -31,18 +31,7 @@ apply_trx_ws(void*                    recv_ctx,
     {
         try
         {
-            if (trx.is_toi())
-            {
-                log_debug << "Executing TO isolated action: " << trx;
-            }
-
             gu_trace(trx.apply(recv_ctx, apply_cb, meta));
-
-            if (trx.is_toi())
-            {
-                log_debug << "Done executing TO isolated action: "
-                         << trx.global_seqno();
-            }
             break;
         }
         catch (galera::ApplyException& e)
@@ -492,6 +481,12 @@ void galera::ReplicatorSMM::apply_trx(void* recv_ctx, TrxHandle* trx)
     wsrep_trx_meta_t meta = {{state_uuid_, trx->global_seqno() },
                              trx->depends_seqno()};
 
+    if (trx->is_toi())
+    {
+        log_debug << "Executing TO isolated action: " << *trx;
+        st_.mark_unsafe();
+    }
+
     gu_trace(apply_trx_ws(recv_ctx, apply_cb_, commit_cb_, *trx, meta));
     /* at this point any exception in apply_trx_ws() is fatal, not
      * catching anything. */
@@ -540,6 +535,13 @@ void galera::ReplicatorSMM::apply_trx(void* recv_ctx, TrxHandle* trx)
 
     apply_monitor_.leave(ao);
 
+    if (trx->is_toi())
+    {
+        log_debug << "Done executing TO isolated action: "
+                  << trx->global_seqno();
+        st_.mark_safe();
+    }
+
     trx->set_exit_loop(exit_loop);
 }
 
@@ -1354,6 +1356,14 @@ void galera::ReplicatorSMM::process_trx(void* recv_ctx, TrxHandle* trx)
     assert(trx->depends_seqno() == -1);
     assert(trx->state() == TrxHandle::S_REPLICATING);
 
+    // If the SST has been canceled, then ignore any other
+    // incoming transactions, as the node should be shutting down
+    if (sst_state_ == SST_CANCELED)
+    {
+        log_info << "Ignorng trx(" << trx->global_seqno() << ") due to SST failure";
+        return;
+    }
+
     wsrep_status_t const retval(cert_and_catch(trx));
 
     switch (retval)
@@ -1783,10 +1793,10 @@ wsrep_seqno_t galera::ReplicatorSMM::pause()
 
 void galera::ReplicatorSMM::resume()
 {
-    assert(pause_seqno_ != WSREP_SEQNO_UNDEFINED);
     if (pause_seqno_ == WSREP_SEQNO_UNDEFINED)
     {
-        gu_throw_error(EALREADY) << "tried to resume unpaused provider";
+        log_warn << "tried to resume unpaused provider";
+        return;
     }
 
     st_.set(state_uuid_, WSREP_SEQNO_UNDEFINED, safe_to_bootstrap_);
