@@ -10,6 +10,8 @@
 #include <gu_logger.hpp>
 #include <gu_throw.hpp>
 #include <gu_progress.hpp>
+#include <gu_hexdump.hpp>
+#include <gu_hash.h>
 
 #include <cassert>
 
@@ -669,7 +671,7 @@ namespace gcache
             }
             else
             {
-                log_warn << "Skipped GCache ring buffer recovery: could not "
+                log_info << "Skipped GCache ring buffer recovery: could not "
                     "determine history UUID.";
             }
         }
@@ -693,11 +695,12 @@ namespace gcache
         size_t collision_count(0);
         int64_t erase_up_to(-1);
         uint8_t* segment_start(start_);
+        uint8_t* segment_end(end_ - sizeof(BufferHeader));
 
         /* start at offset (first segment) if we know it and it is valid */
         if (offset >= 0)
         {
-            if (start_ + offset + 2*sizeof(BufferHeader) < end_)
+            if (start_ + offset + sizeof(BufferHeader) < segment_end)
                 /* we know exaclty where the first segment starts */
                 segment_start = start_ + offset;
             else
@@ -705,11 +708,8 @@ namespace gcache
                 segment_scans = 1;
         }
 
-        gu::Progress<ptrdiff_t> progress("GCache::RingBuffer initial scan ",
-                                         " bytes",
-                                         "PT5S", /* 5 sec */
-                                         1<<22,  /* 4Mb */
-                                         end_ - start_);
+        gu::Progress<ptrdiff_t> progress("GCache::RingBuffer initial scan",
+                                         " bytes", end_ - start_, 1<<22 /*4Mb*/);
 
         while (segment_scans < 2)
         {
@@ -720,7 +720,7 @@ namespace gcache
 
 #define GCACHE_SCAN_BUFFER_TEST                                 \
             (BH_test(bh) && bh->size > 0 &&                     \
-             ptr + bh->size + sizeof(BufferHeader) <= end_ &&   \
+             ptr + bh->size <= segment_end &&                   \
              BH_test(BH_cast(ptr + bh->size)))
 
             while (GCACHE_SCAN_BUFFER_TEST)
@@ -745,22 +745,69 @@ namespace gcache
                     {
                         collision_count++;
 
-                        log_info <<"Attempt to reuse the same seqno: " << seqno_g
-                                 << ". New ptr = " << static_cast<void*>(bh+1)
-                                 << ", previous ptr = " << res.first->second;
+                        /* compare two buffers */
+                        const void* const old_ptr(res.first->second);
+                        BufferHeader* const old_bh
+                            (old_ptr ? ptr2BH(old_ptr) : NULL);
+
+                        bool const same_meta(NULL != old_bh &&
+                            bh->seqno_g == old_bh->seqno_g  &&
+                            bh->size    == old_bh->size     &&
+                            bh->flags   == old_bh->flags);
+
+                        const void* const new_ptr(static_cast<void*>(bh+1));
+
+                        uint8_t cs_old[16] = { 0, };
+                        uint8_t cs_new[16] = { 0, };
+                        if (same_meta)
+                        {
+                            gu_fast_hash128(old_ptr,
+                                            old_bh->size - sizeof(BufferHeader),
+                                            cs_old);
+                            gu_fast_hash128(new_ptr,
+                                            bh->size - sizeof(BufferHeader),
+                                            cs_new);
+                        }
+
+                        bool const same_data(same_meta &&
+                                             !::memcmp(cs_old, cs_new,
+                                                       sizeof(cs_old)));
+                        std::ostringstream msg;
+
+                        msg << "Attempt to reuse the same seqno: " << seqno_g
+                            << ". New ptr = " << new_ptr << ", " << bh
+                            << ", cs: " << gu::Hexdump(cs_new, sizeof(cs_new))
+                            << ", previous ptr = " << res.first->second;
+
                         empty_buffer(bh); // this buffer is unusable
                         assert(BH_is_released(bh));
 
-                        if (res.first->second != NULL)
+                        if (old_bh != NULL)
                         {
-                            BufferHeader* b(ptr2BH(res.first->second));
-                            empty_buffer(b);
-                            assert(BH_is_released(b));
-                            res.first->second = NULL; // mark entry as invalid
-                            // but don't remove from the map to block this seqno
+                            msg << ", " << old_bh << ", cs: "
+                                << gu::Hexdump(cs_old,sizeof(cs_old));
+
+                            if (!same_data) // no way to choose which is correct
+                            {
+                                empty_buffer(old_bh);
+                                assert(BH_is_released(old_bh));
+                                res.first->second = NULL;
+                                /* mark entry as invalid, but don't remove from
+                                 * the map to block this seqno */
+
+                                if (erase_up_to < seqno_g) erase_up_to = seqno_g;
+                            }
                         }
 
-                        if (erase_up_to < seqno_g) erase_up_to = seqno_g;
+                        log_info << msg.str();
+
+                        if (same_data) {
+                            log_info << "Contents are the same, discarding "
+                                     << new_ptr;
+                        } else {
+                            assert(NULL == res.first->second);
+                            log_info << "Contents differ. Discarding both.";
+                        }
                     }
                 }
 
@@ -771,15 +818,15 @@ namespace gcache
 
             if (!BH_is_clear(bh))
             {
-                if (start_ == segment_start)
+                if (start_ == segment_start && ptr != first_)
                 {
                     log_warn << "Failed to scan the last segment to the end. "
                             "Last events may be missing. Last recovered event: "
                              << gid_ << ':' << seqno_max;
                 }
 
-                /* do best effort */
-                BH_clear(bh);
+                /* end of file, do best effort */
+                if (end_ - sizeof(BufferHeader) == segment_end) BH_clear(bh);
             }
 
             if (offset > 0 && segment_start == start_ + offset)
@@ -788,6 +835,7 @@ namespace gcache
                 assert(1 == segment_scans);
                 first_ = segment_start;
                 size_trail_ = end_ - ptr;
+                segment_end = segment_start;
                 segment_start = start_;
             }
             else if (offset < 0 && segment_start == start_)
@@ -844,7 +892,7 @@ namespace gcache
                     /* first (end) segment was scanned last, estimate trail */
                     size_trail_ = end_ - ptr;
                 }
-                else if (offset > 0 && next_ >= first_)
+                else if (offset > 0 && next_ > first_)
                 {
                     size_trail_ = 0;
                 }
@@ -867,9 +915,8 @@ namespace gcache
 
         if (!seqno2ptr_.empty())
         {
-            assert(first_ != next_);
-            assert(next_ < first_ || size_trail_ == 0);
-            assert(next_ > first_ || size_trail_ >  0);
+            assert(next_ <= first_ || size_trail_ == 0);
+            assert(next_ >  first_ || size_trail_ >  0);
 
             /* find the last gapless seqno sequence */
             seqno2ptr_t::reverse_iterator r(seqno2ptr_.rbegin());
@@ -918,7 +965,7 @@ namespace gcache
                 /* clear up seqno2ptr map */
                 for (; r != seqno2ptr_.rend(); ++r)
                 {
-                    empty_buffer(ptr2BH(r->second));
+                    if (r->second) empty_buffer(ptr2BH(r->second));
                 }
                 seqno2ptr_.erase(seqno2ptr_.begin(), seqno2ptr_.find(seqno_min));
             }
@@ -940,8 +987,8 @@ namespace gcache
 
             /* trim next_: start with the last seqno and scan forward up to the
              * current next_. Update to the end of the last non-empty buffer. */
-            BufferHeader* last_bh(NULL);
             bh = ptr2BH(seqno2ptr_[seqno_max]);
+            BufferHeader* last_bh(bh);
             while (bh != BH_cast(next_))
             {
                 if (gu_likely(bh->size) > 0)
@@ -958,7 +1005,6 @@ namespace gcache
                     bh = BH_cast(start_); // rollover
                 }
             }
-            assert(last_bh);
             next_ = reinterpret_cast<uint8_t*>(BH_next(last_bh));
 
             /* at this point we must have at least one seqno'd buffer */
@@ -972,11 +1018,8 @@ namespace gcache
 
             /* now discard all the locked-in buffers (see seqno_reset()) */
             gu::Progress<size_t> progress(
-                "GCache::RingBuffer unused buffers scan ",
-                " bytes",
-                "PT5S", /* 5 sec */
-                1<<22,  /* 4Mb   */
-                size_used_);
+                "GCache::RingBuffer unused buffers scan",
+                " bytes", size_used_, 1<<22 /* 4Mb */);
 
             size_t total(0);
             size_t locked(0);
