@@ -1645,3 +1645,31 @@ gcs_group_get_status (gcs_group_t* group, gu::Status& status)
 
     status.insert("desync_count", gu::to_string(desync_count));
 }
+
+void
+gcs_group_fetch_pfs_info(
+    const gcs_group_t* group,
+    wsrep_node_info_t* entries,
+    uint32_t size)
+{
+    if (size < (uint32_t) group->num)
+      return;
+
+    for (uint i= 0; i < (uint32_t) group->num; i++)
+    {
+        const gcs_node_t& node(group->nodes[i]);
+
+        strncpy(entries[i].host_name, node.name, WSREP_HOSTNAME_LENGTH);
+
+        strncpy(entries[i].uuid, node.id, WSREP_UUID_STR_LEN);
+
+        strncpy(
+            entries[i].status,
+            gcs_node_state_to_str(node.status),
+            WSREP_STATUS_LENGTH);
+
+        entries[i].local_index = i;
+        entries[i].segment = node.segment;
+    }
+}
+
