@@ -216,6 +216,10 @@ eval_log_error () {
 
   #echo "Running mysqld: [$cmd]"
   eval "$cmd"
+  ret=$?
+  if [ $ret > 0 ]; then
+    exit $ret
+  fi
 }
 
 shell_quote_string() {
