@@ -25,7 +25,6 @@
 # related to role/username/password/etc...
 #
 . $(dirname $0)/wsrep_sst_common
-WSREP_LOG_DEBUG=$(parse_cnf sst wsrep-log-debug "")
 
 #-------------------------------------------------------------------------------
 #
@@ -37,6 +36,10 @@ ealgo=""
 ekey=""
 ekeyfile=""
 encrypt=0
+ieopts=""
+xbstreameopts=""
+xbstreameopts_sst=""
+xbstreameopts_other=""
 
 nproc=1
 ecode=0
@@ -48,11 +51,13 @@ tca=""
 tcert=""
 tkey=""
 sockopt=""
+ncsockopt=""
 progress=""
 ttime=0
 totime=0
 lsn=""
 ecmd=""
+ecmd_other=""
 rlimit=""
 stagemsg="${WSREP_SST_OPT_ROLE}"
 cpat=""
@@ -100,6 +105,10 @@ ssl_cert=""
 ssl_ca=""
 ssl_key=""
 
+# thread options
+backup_threads=-1
+encrypt_threads=-1
+
 # Required for backup locks
 # For backup locks it is 1 sent by joiner
 # 5.6.21 PXC and later can't donate to an older joiner
@@ -123,6 +132,12 @@ XB_GTID_INFO_FILE="xtrabackup_galera_info"
 XB_GTID_INFO_FILE_PATH="${DATA}/${XB_GTID_INFO_FILE}"
 IST_FILE="xtrabackup_ist"
 
+# Used to send a file containing info about the SST
+# Extend this if you want to send additional information
+# from the donor to the joiner
+SST_INFO_FILE="sst_info"
+
+# Setting the path for ss and ip
 # ss: is utility to investigate socket, ip for network routes.
 export PATH="/usr/sbin:/sbin:$PATH"
 
@@ -158,10 +173,14 @@ timeit()
 }
 
 #
-# get encryption keys.
+# Configures the commands for encrypt=1
+# Sets up the parameters (algo/keys) for XB-based encryption
+# If the version of PXB is >= 2.4.7, changes the XB parameters
+# If the version of PXB is < 2.4.7, use xbcrypt instead
+#
 get_keys()
 {
-    # $encrypt -eq 1 is for internal purposes only
+    # $encrypt -eq -1 is for internal purposes only
     if [[ $encrypt -ge 2 || $encrypt -eq -1 ]]; then
         return
     fi
@@ -175,9 +194,11 @@ get_keys()
     fi
 
     if [[ $sfmt == 'tar' ]]; then
-        wsrep_log_error "NOTE: Xtrabackup-based encryption - encrypt=1 - cannot be enabled with tar format"
-        encrypt=-1
-        return
+        wsrep_log_error "******************* FATAL ERROR ********************** "
+        wsrep_log_error "* Xtrabackup-based encryption (encrypt = 1) cannot be  "
+        wsrep_log_error "* enabled with the tar format.                         "
+        wsrep_log_error "****************************************************** "
+        exit 22
     fi
 
     wsrep_log_debug "Xtrabackup based encryption enabled in my.cnf"
@@ -196,18 +217,53 @@ get_keys()
         exit 3
     fi
 
-    if [[ -z $ekey ]]; then
-        ecmd="xbcrypt --encrypt-algo=$ealgo --encrypt-key-file=$ekeyfile"
-    else
+    if [[ -n "$ekey" ]]; then
         wsrep_log_warning "Using the 'encrypt-key' option causes the encryption key"
         wsrep_log_warning "to be set via the command-line and is considered insecure."
         wsrep_log_warning "It is recommended to use the 'encrypt-key-file' option instead."
-
-        ecmd="xbcrypt --encrypt-algo=$ealgo --encrypt-key=$ekey"
     fi
 
+    local readonly encrypt_opts=""
+    if [[ -z $ekey ]]; then
+        encrypt_opts=" --encrypt-key-file=$ekeyfile "
+    else
+        encrypt_opts=" --encrypt-key=$ekey "
+    fi
+
+    # Setup the command for all non-SST transfers
+    # Encryption is done by xbcrypt for all other (non-SST) transfers
+    #
+    ecmd_other="xbcrypt --encrypt-algo=$ealgo $encrypt_opts "
     if [[ "$WSREP_SST_OPT_ROLE" == "joiner" ]]; then
-        ecmd+=" -d"
+        # Decryption is done by xbcrypt for all other (non-SST) transfers
+        ecmd_other+=" -d"
+    fi
+
+    #
+    # PXB version >= 2.4.7 added encryption support directly in PXB
+    #
+    if check_for_version $XB_VERSION "2.4.7"; then
+        # ensure that ecmd is clear because SST encryption
+        # goes through xtrabackup, not a separate program
+        ecmd=""
+
+        if [[ "$WSREP_SST_OPT_ROLE" == "joiner" ]]; then
+            # Decryption is done by xbstream for SST
+            xbstreameopts_sst=" --decrypt=$ealgo $encrypt_opts --encrypt-threads=$encrypt_threads "
+            xbstreameopts_other=""
+            ieopts=""
+        else
+            # Encryption is done by xtrabackup for SST
+            xbstreameopts_sst=""
+            xbstreameopts_other=""
+            ieopts=" --encrypt=$ealgo $encrypt_opts --encrypt-threads=$encrypt_threads "
+        fi
+    else
+        #
+        # For older versions, use xbcrypt for SST encryption/decryption
+        # (same as for non-SST transfers)
+        #
+        ecmd=$ecmd_other
     fi
 
     stagemsg+="-XB-Encrypted"
@@ -349,12 +405,13 @@ get_transfer()
         wsrep_log_debug "Using netcat as streamer"
         if [[ "$WSREP_SST_OPT_ROLE"  == "joiner" ]]; then
             if nc -h 2>&1 | grep -q ncat; then
-                tcmd="nc -l ${TSST_PORT}"
-            else
-                tcmd="nc -dl ${TSST_PORT}"
+                tcmd="nc $ncsockopt -l ${TSST_PORT}"
+            else 
+                tcmd="nc $ncsockopt -dl ${TSST_PORT}"
             fi
         else
-            tcmd="nc ${REMOTEIP} ${TSST_PORT}"
+            # netcat doesn't understand [] around IPv6 address
+            tcmd="nc ${REMOTEIP//[\[\]]/} ${TSST_PORT}"
         fi
     else
         tfmt='socat'
@@ -480,38 +537,6 @@ get_transfer()
     fi
 }
 
-#
-# Converts an input string into a boolean "on", "off"
-# Converts:
-#   "on", "true", "1" -> "on" (case insensitive)
-#   "off", "false", "0" -> "off" (case insensitive)
-# All other values : -> default_value
-#
-# 1st param: input value
-# 2nd param: default value
-normalize_boolean()
-{
-    local input_value=$1
-    local default_value=$2
-    local return_value=$default_value
-
-    if [[ "$input_value" == "1" ]]; then
-        return_value="on"
-    elif [[ "$input_value" =~ ^[Oo][Nn]$ ]]; then
-        return_value="on"
-    elif [[ "$input_value" =~ ^[Tt][Rr][Uu][Ee]$ ]]; then
-        return_value="on"
-    elif [[ "$input_value" == "0" ]]; then
-        return_value="off"
-    elif [[ "$input_value" =~ ^[Oo][Ff][Ff]$ ]]; then
-        return_value="off"
-    elif [[ "$input_value" =~ ^[Ff][Aa][Ll][Ss][Ee]$ ]]; then
-        return_value="off"
-    fi
-
-    echo $return_value
-}
-
 #
 # read the sst specific options.
 read_cnf()
@@ -523,6 +548,7 @@ read_cnf()
     tkey=$(parse_cnf sst tkey "")
     encrypt=$(parse_cnf sst encrypt 0)
     sockopt=$(parse_cnf sst sockopt "")
+    ncsockopt=$(parse_cnf sst ncsockopt "")
     progress=$(parse_cnf sst progress "")
     rebuild=$(parse_cnf sst rebuild 0)
     ttime=$(parse_cnf sst time 0)
@@ -574,6 +600,8 @@ read_cnf()
     pxc_encrypt_cluster_traffic=$(parse_cnf mysqld pxc-encrypt-cluster-traffic "")
     pxc_encrypt_cluster_traffic=$(normalize_boolean "$pxc_encrypt_cluster_traffic" "off")
 
+    ssl_dhparams=$(parse_cnf sst ssl-dhparams "")
+
     rlimit=$(parse_cnf sst rlimit "")
     uextra=$(parse_cnf sst use-extra 0)
     iopts=$(parse_cnf sst inno-backup-opts "")
@@ -584,6 +612,23 @@ read_cnf()
     ssystag=$(parse_cnf mysqld_safe syslog-tag "${SST_SYSLOG_TAG:-}")
     ssystag+="-"
 
+    # thread options
+    encrypt_threads=$(parse_cnf sst encrypt-threads -1)
+    if [[ $encrypt_threads -eq -1 ]]; then
+        encrypt_threads=$(parse_cnf sst encrypt_threads -1)
+    fi
+    if [[ $encrypt_threads -le 0 ]]; then
+        encrypt_threads=4
+    fi
+
+    backup_threads=$(parse_cnf sst backup-threads -1)
+    if [[ $backup_threads -eq -1 ]]; then
+        backup_threads=$(parse_cnf sst backup_threads -1)
+    fi
+    if [[ $backup_threads -le 0 ]]; then
+        backup_threads=4
+    fi
+
     if [[ $ssyslog -ne -1 ]]; then
         if $MY_PRINT_DEFAULTS -c $WSREP_SST_OPT_CONF mysqld_safe | tr '_' '-' | grep -q -- "--syslog"; then
             ssyslog=1
@@ -608,8 +653,8 @@ read_cnf()
     fi
 
     # Setup # of threads (if not already specified)
-    if ! [[ "$iopts" =~ --parallel= ]]; then
-        iopts+=" --parallel=4"
+    if [[ ! "$iopts" =~ --parallel= ]]; then
+        iopts+=" --parallel=$backup_threads"
     fi
 
     # Buildup the list of files to keep in the datadir
@@ -627,12 +672,17 @@ read_cnf()
         # Add the path to each subdirectory, this will ensure that
         # the path to the keyring is kept
         local CURRENT_DIR=$(dirname "$KEYRING_DIR/xx")
-        while [[ $CURRENT_DIR != "." && $CURRENT_DIR != "/" && $CURRENT_DIR != $DATA_TEMP ]]; do
+        while [[ $CURRENT_DIR != "." && $CURRENT_DIR != "/" && $CURRENT_DIR != "$DATA_TEMP" ]]; do
             cpat+="\|${CURRENT_DIR}$"
             CURRENT_DIR=$(dirname "$CURRENT_DIR")
         done
     fi
 
+    # Retry the connection 30 times (at 1-second intervals)
+    if [[ ! "$sockopt" =~ retry= ]]; then
+        sockopt+=",retry=30"
+    fi
+
 }
 
 #
@@ -684,15 +734,19 @@ adjust_progress()
 }
 
 #
-# data is stream from donor to joiner using xbstream program.
+# Fills in strmcmd, which holds the command used for streaming
+#
+# Note:
+#   This code creates a command that uses FILE_TO_STREAM
+#
 get_stream()
 {
     if [[ $sfmt == 'xbstream' ]]; then
         wsrep_log_debug "Streaming with xbstream"
         if [[ "$WSREP_SST_OPT_ROLE"  == "joiner" ]]; then
-            strmcmd="xbstream -x"
+            strmcmd="xbstream \$xbstreameopts -x"
         else
-            strmcmd="xbstream -c \${FILE_TO_STREAM}"
+            strmcmd="xbstream \$xbstreameopts -c \${FILE_TO_STREAM}"
         fi
     else
         sfmt="tar"
@@ -764,12 +818,6 @@ cleanup_joiner()
     exit $estatus
 }
 
-check_pid()
-{
-    local pid_file="$1"
-    [ -r "$pid_file" ] && ps -p $(cat "$pid_file") >/dev/null 2>&1
-}
-
 cleanup_donor()
 {
     # Since this is invoked just after exit NNN
@@ -809,34 +857,36 @@ cleanup_donor()
     rm -rf "${KEYRING_DIR}/${XB_DONOR_KEYRING_FILE}" || true
 
     exit $estatus
+
 }
 
 setup_ports()
 {
-    if [[ "$WSREP_SST_OPT_ROLE"  == "donor" ]]; then
-        SST_PORT=$(echo $WSREP_SST_OPT_ADDR | awk -F '[:/]' '{ print $2 }')
-        REMOTEIP=$(echo $WSREP_SST_OPT_ADDR | awk -F ':' '{ print $1 }')
-        lsn=$(echo $WSREP_SST_OPT_ADDR | awk -F '[:/]' '{ print $4 }')
-        sst_ver=$(echo $WSREP_SST_OPT_ADDR | awk -F '[:/]' '{ print $5 }')
+    if [[ "$WSREP_SST_OPT_ROLE"  == "donor" ]];then
+        SST_PORT=$WSREP_SST_OPT_PORT
+        REMOTEIP=$WSREP_SST_OPT_HOST
+        lsn=$(echo $WSREP_SST_OPT_PATH | awk -F '[/]' '{ print $2 }')
+        sst_ver=$(echo $WSREP_SST_OPT_PATH | awk -F '[/]' '{ print $3 }')
     else
-        SST_PORT=$(echo ${WSREP_SST_OPT_ADDR} | awk -F ':' '{ print $2 }')
+        SST_PORT=$WSREP_SST_OPT_PORT
     fi
 }
 
-#
-# waits ~10 seconds for nc to open the port and then reports ready
+# waits ~1 minute for nc/socat to open the port and then reports ready
 # (regardless of timeout)
 wait_for_listen()
 {
-    local PORT=$1
-    local ADDR=$2
+    local HOST=$1
+    local PORT=$2
     local MODULE=$3
-    for i in {1..50}
+
+    for i in {1..300}
     do
         ss -p state listening "( sport = :$PORT )" | grep -qE 'socat|nc' && break
         sleep 0.2
     done
-    echo "ready ${ADDR}/${MODULE}//$sst_ver"
+
+    echo "ready ${HOST}:${PORT}/${MODULE}//$sst_ver"
 }
 
 #
@@ -874,6 +924,7 @@ check_extra()
 # 2nd param: msg
 # 3rd param: tmt : timeout
 # 4th param: checkf : file check
+#   If this is -2, skip the file checks (but still check the return code)
 #   If this is -1, skip all error checking
 #   If this is 0, do nothing (no additional checks)
 #   If this is 1, check to see if the gtid info file exists
@@ -932,6 +983,11 @@ recv_data_from_donor_to_joiner()
         fi
     done
 
+    if [[ $checkf -eq -2 ]]; then
+        # no file checking
+        return
+    fi
+
     if [[ $checkf -eq 1 && ! -r "${XB_GTID_INFO_FILE_PATH}" ]]; then
         # this message should cause joiner to abort
         wsrep_log_error "******************* FATAL ERROR ********************** "
@@ -951,11 +1007,15 @@ recv_data_from_donor_to_joiner()
         wsrep_log_error "****************************************************** "
         exit 32
     fi
-
 }
 
 #
 # Process to send data from DONOR to JOINER
+#
+# Parameters:
+#   1 : dir - the base directory (paths are based on this)
+#   2 : msg - descriptive message
+#
 send_data_from_donor_to_joiner()
 {
     local dir=$1
@@ -1002,7 +1062,7 @@ normalize_version()
 # Compares two version strings
 # The first parameter is the version to be checked
 # The second parameter is the minimum version required
-# Returns 1 (failure) if $1 >= $2, 0 (success) otherwise
+# Returns 0 (success) if $1 >= $2, 1 (failure) otherwise
 check_for_version()
 {
     local local_version_str="$( normalize_version $1 )"
@@ -1059,6 +1119,38 @@ initialize_tmpdir()
 }
 
 
+#
+# Parses the passed in config file and returns the option in the
+# specified group.
+#
+# 1st param: source_path : path the the source file
+# 2nd param: group : name of the config file section, e.g. mysqld
+# 3rd param: var : name of the variable in the section, e.g. server-id
+# 4th param: - : default value for the param
+#
+parse_sst_info()
+{
+    local source_path=$1
+    local group=$2
+    local var=$3
+    local reval=""
+
+    # print the default settings for given group using my_print_default.
+    # normalize the variable names specified in cnf file (user can use _ or -
+    # for example log-bin or log_bin) then grep for needed variable
+    # finally get the variable value (if variables has been specified
+    # multiple time use the last value only)
+
+    reval=$($MY_PRINT_DEFAULTS -c "$source_path" $group | awk -F= '{if ($1 ~ /_/) { gsub(/_/,"-",$1); print $1"="$2 } else { print $0 }}' | grep -- "--$var=" | cut -d= -f2- | tail -1)
+
+    # use default if we haven't found a value
+    if [[ -z $reval ]]; then
+        [[ -n $4 ]] && reval=$4
+    fi
+
+    echo $reval
+}
+
 #-------------------------------------------------------------------------------
 #
 # Step-4: Main workload logic starts here.
@@ -1099,6 +1191,9 @@ fi
 
 wsrep_log_debug "The $INNOBACKUPEX_BIN version is $XB_VERSION"
 
+# Get our MySQL version
+MYSQL_VERSION=$($(dirname $0)/mysqld --version 2>&1 | grep -oe '[0-9]\.[0-9][\.0-9]*' | head -n1)
+
 rm -f "${XB_GTID_INFO_FILE_PATH}"
 
 #
@@ -1195,19 +1290,31 @@ if [[ $ssyslog -eq 1 ]]; then
             logger -p daemon.err -t ${ssystag}wsrep-sst-$WSREP_SST_OPT_ROLE "$@"
         }
 
+        wsrep_log_warning()
+        {
+            logger -p daemon.warn -t ${ssystag}wsrep-sst-$WSREP_SST_OPT_ROLE "$@"
+        }
+
         wsrep_log_info()
         {
             logger -p daemon.info -t ${ssystag}wsrep-sst-$WSREP_SST_OPT_ROLE "$@"
         }
 
+        wsrep_log_debug()
+        {
+            if [[ -n "$WSREP_LOG_DEBUG" ]]; then
+                logger -p daemon.debug -t ${ssystag}wsrep-sst-$WSREP_SST_OPT_ROLE "$@"
+            fi
+        }
+
         INNOAPPLY="${INNOBACKUPEX_BIN} $disver $iapts --prepare --binlog-info=ON \$rebuildcmd \$keyringapplyopt --target-dir=\${DATA} 2>&1  | logger -p daemon.err -t ${ssystag}innobackupex-apply "
-        INNOMOVE="${INNOBACKUPEX_BIN} --defaults-file=${WSREP_SST_OPT_CONF} --datadir=\${TDATA} $disver $impts  --move-back --binlog-info=ON --force-non-empty-directories --target-dir=\${DATA} 2>&1 | logger -p daemon.err -t ${ssystag}innobackupex-move "
-        INNOBACKUP="${INNOBACKUPEX_BIN} --defaults-file=${WSREP_SST_OPT_CONF} $disver $iopts \$INNOEXTRA \$keyringbackupopt --backup --galera-info  --binlog-info=ON --stream=\$sfmt --target-dir=\$itmpdir 2> >(logger -p daemon.err -t ${ssystag}innobackupex-backup)"
+        INNOMOVE="${INNOBACKUPEX_BIN} --defaults-file=${WSREP_SST_OPT_CONF} --datadir=\${TDATA} --defaults-group=mysqld${WSREP_SST_OPT_CONF_SUFFIX} $disver $impts  --move-back --binlog-info=ON --force-non-empty-directories --target-dir=\${DATA} 2>&1 | logger -p daemon.err -t ${ssystag}innobackupex-move "
+        INNOBACKUP="${INNOBACKUPEX_BIN} --defaults-file=${WSREP_SST_OPT_CONF} --defaults-group=mysqld${WSREP_SST_OPT_CONF_SUFFIX} $disver $iopts \$ieopts \$INNOEXTRA \$keyringbackupopt --backup --galera-info  --binlog-info=ON --stream=\$sfmt --target-dir=\$itmpdir 2> >(logger -p daemon.err -t ${ssystag}innobackupex-backup)"
     fi
 else
     INNOAPPLY="${INNOBACKUPEX_BIN} $disver $iapts --prepare --binlog-info=ON \$rebuildcmd \$keyringapplyopt --target-dir=\${DATA} &>\${DATA}/innobackup.prepare.log"
     INNOMOVE="${INNOBACKUPEX_BIN} --defaults-file=${WSREP_SST_OPT_CONF}  --defaults-group=mysqld${WSREP_SST_OPT_CONF_SUFFIX} --datadir=\${TDATA} $disver $impts  --move-back --binlog-info=ON --force-non-empty-directories --target-dir=\${DATA} &>\${DATA}/innobackup.move.log"
-    INNOBACKUP="${INNOBACKUPEX_BIN} --defaults-file=${WSREP_SST_OPT_CONF}  --defaults-group=mysqld${WSREP_SST_OPT_CONF_SUFFIX} $disver $iopts \$INNOEXTRA \$keyringbackupopt --backup --galera-info --binlog-info=ON --stream=\$sfmt --target-dir=\$itmpdir 2>\${DATA}/innobackup.backup.log"
+    INNOBACKUP="${INNOBACKUPEX_BIN} --defaults-file=${WSREP_SST_OPT_CONF}  --defaults-group=mysqld${WSREP_SST_OPT_CONF_SUFFIX} $disver $iopts \$ieopts \$INNOEXTRA \$keyringbackupopt --backup --galera-info --binlog-info=ON --stream=\$sfmt --target-dir=\$itmpdir 2>\${DATA}/innobackup.backup.log"
 fi
 
 #
@@ -1221,6 +1328,24 @@ then
     # signal handler for cleanup-based-exit.
     trap cleanup_donor EXIT
 
+    initialize_tmpdir
+
+    # main temp directory for SST (non-XB) related files
+    donor_tmpdir=$(mktemp -p "${tmpdirbase}" -dt donor_tmp_XXXXXXXX)
+
+    # Create the SST info file
+    # This file contains SST information that is passed from the
+    # donor to the joiner.
+    #
+    # Add more parameters to the file here as needed
+    # This file has the same format as a cnf file.
+    #
+    sst_info_file_path="${donor_tmpdir}/${SST_INFO_FILE}"
+    echo "[sst]" > "$sst_info_file_path"
+    echo "galera-gtid=$WSREP_SST_OPT_GTID" >> "$sst_info_file_path"
+    echo "binlog-name=$(basename "$WSREP_SST_OPT_BINLOG")" >> "$sst_info_file_path"
+    echo "mysql-version=$MYSQL_VERSION" >> "$sst_info_file_path"
+
     #
     # SST is not needed. IST would suffice. By-pass SST.
     if [ $WSREP_SST_OPT_BYPASS -eq 0 ]
@@ -1234,8 +1359,6 @@ then
             exit 93
         fi
 
-        initialize_tmpdir
-
         # main temp directory for xtrabackup (target-dir)
         itmpdir=$(mktemp -p "${tmpdirbase}" -dt donor_xb_XXXXXXXX)
 
@@ -1255,20 +1378,24 @@ then
         check_extra
 
         ttcmd="$tcmd"
-        if [[ $encrypt -eq 1 ]]; then
-            if [[ -n $scomp ]]; then
-                tcmd=" \$ecmd | $scomp | $tcmd "
-            else
-                tcmd=" \$ecmd | $tcmd "
-            fi
-        elif [[ -n $scomp ]]; then
+
+        # Add compression to the head of the stream (if specified)
+        if [[ -n "$scomp" ]]; then
             tcmd=" $scomp | $tcmd "
         fi
+        # Add encryption to the head of the stream (if specified)
+        if [[ $encrypt -eq 1 && -n "$ecmd_other" ]]; then
+            tcmd=" \$ecmd_other | $tcmd "
+        fi
+        if [[ $encrypt -eq 1 ]]; then
+            xbstreameopts=$xbstreameopts_other
+        fi
 
-        wsrep_log_info "Streaming GTID file before SST"
-        echo "${WSREP_SST_OPT_GTID}" > "${XB_GTID_INFO_FILE_PATH}"
-        FILE_TO_STREAM=$XB_GTID_INFO_FILE
-        send_data_from_donor_to_joiner $DATA "${stagemsg}-gtid"
+        # Before the real SST,send the sst-info
+        wsrep_log_debug "Streaming SST meta-info file before SST"
+
+        FILE_TO_STREAM=$SST_INFO_FILE
+        send_data_from_donor_to_joiner "$donor_tmpdir" "${stagemsg}-sst-info"
 
         if [[ -n $keyring ]]; then
             # Verify that encryption is being used
@@ -1313,11 +1440,13 @@ then
         if [[ -n $scomp ]]; then
             tcmd="$scomp | $tcmd"
         fi
-
         # Add encryption to the head of the stream (if specified)
-        if [[ $encrypt -eq 1 ]]; then
+        if [[ $encrypt -eq 1 && -n "$ecmd" ]]; then
             tcmd=" \$ecmd | $tcmd "
         fi
+        if [[ $encrypt -eq 1 ]]; then
+            xbstreameopts=$xbstreameopts_sst
+        fi
 
         set +e
         timeit "${stagemsg}-SST" "$INNOBACKUP | $tcmd; RC=( "\${PIPESTATUS[@]}" )"
@@ -1343,24 +1472,24 @@ then
 
         wsrep_log_info "Bypassing SST. Can work it through IST"
         echo "continue" # now server can resume updating data
-        echo "${WSREP_SST_OPT_GTID}" > "${XB_GTID_INFO_FILE_PATH}"
-        echo "1" > "${DATA}/${IST_FILE}"
-
+        echo "1" > "${donor_tmpdir}/${IST_FILE}"
         get_keys
-        if [[ $encrypt -eq 1 ]]; then
-            if [[ -n $scomp ]]; then
-                tcmd=" \$ecmd | $scomp | $tcmd "
-            else
-                tcmd=" \$ecmd | $tcmd "
-            fi
-        elif [[ -n $scomp ]]; then
+        # Add compression to the head of the stream (if specified)
+        if [[ -n "$scomp" ]]; then
             tcmd=" $scomp | $tcmd "
         fi
+        # Add encryption to the head of the stream (if specified)
+        if [[ $encrypt -eq 1 && -n "$ecmd_other" ]]; then
+            tcmd=" \$ecmd_other | $tcmd "
+        fi
+        if [[ $encrypt -eq 1 ]]; then
+            xbstreameopts=$xbstreameopts_other
+        fi
+
         strmcmd+=" \${IST_FILE}"
 
-	FILE_TO_STREAM=$XB_GTID_INFO_FILE
-        send_data_from_donor_to_joiner $DATA "${stagemsg}-IST"
-
+        FILE_TO_STREAM=$SST_INFO_FILE
+        send_data_from_donor_to_joiner "$donor_tmpdir" "${stagemsg}-IST"
     fi
 
     echo "done ${WSREP_SST_OPT_GTID}"
@@ -1378,7 +1507,6 @@ then
 
     stagemsg="Joiner-Recv"
 
-    sencrypted=1
     nthreads=1
 
     MODULE="xtrabackup_sst"
@@ -1391,14 +1519,7 @@ then
         rm -f "${KEYRING_DIR}/${XB_DONOR_KEYRING_FILE}"
     fi
 
-    ADDR=${WSREP_SST_OPT_ADDR}
-    if [ -z "${SST_PORT}" ]
-    then
-        SST_PORT=4444
-        ADDR="$(echo ${WSREP_SST_OPT_ADDR} | awk -F ':' '{ print $1 }'):${SST_PORT}"
-    fi
-
-    wait_for_listen ${SST_PORT} ${ADDR} ${MODULE} &
+    wait_for_listen ${WSREP_SST_OPT_HOST} ${WSREP_SST_OPT_PORT:-4444} ${MODULE} &
 
     trap sig_joiner_cleanup HUP PIPE INT TERM
     trap cleanup_joiner EXIT
@@ -1409,21 +1530,78 @@ then
     fi
 
     get_keys
-    if [[ $encrypt -eq 1 && $sencrypted -eq 1 ]]; then
-        if [[ -n $sdecomp ]]; then
-            strmcmd=" $sdecomp | \$ecmd | $strmcmd"
-        else
-            strmcmd=" \$ecmd | $strmcmd"
-        fi
-    elif [[ -n $sdecomp ]]; then
-            strmcmd=" $sdecomp | $strmcmd"
+    if [[ $encrypt -eq 1 && -n "$ecmd_other" ]]; then
+        strmcmd=" \$ecmd_other | $strmcmd"
+    fi
+    if [[ -n $sdecomp ]]; then
+        strmcmd=" $sdecomp | $strmcmd"
+    fi
+    if [[ $encrypt -eq 1 ]]; then
+        xbstreameopts=$xbstreameopts_other
     fi
 
     initialize_tmpdir
-
     STATDIR=$(mktemp -p "${tmpdirbase}" -dt joiner_XXXXXXXX)
-    XB_GTID_INFO_FILE_PATH="${STATDIR}/${XB_GTID_INFO_FILE}"
-    recv_data_from_donor_to_joiner $STATDIR "${stagemsg}-gtid" $stimeout 1
+    sst_file_info_path="${STATDIR}/${SST_INFO_FILE}"
+
+    recv_data_from_donor_to_joiner $STATDIR "${stagemsg}-sst-info" $stimeout -2
+
+    #
+    # Determine which file was received, the GTID or the SST_INFO
+    #
+    if [[ -r "${STATDIR}/${SST_INFO_FILE}" ]]; then
+        #
+        # Extract information from the sst-info file that was just received
+        #
+        XB_GTID_INFO_FILE_PATH="${STATDIR}/${XB_GTID_INFO_FILE}"
+        parse_sst_info "$sst_file_info_path" sst galera-gtid "" > "$XB_GTID_INFO_FILE_PATH"
+
+        DONOR_BINLOGNAME=$(parse_sst_info "$sst_file_info_path" sst binlog-name "")
+        DONOR_MYSQL_VERSION=$(parse_sst_info "$sst_file_info_path" sst mysql-version "")
+
+    elif [[ -r "${STATDIR}/${XB_GTID_INFO_FILE}" ]]; then
+        #
+        # For compatibility, we have received the gtid file
+        #
+        XB_GTID_INFO_FILE_PATH="${STATDIR}/${XB_GTID_INFO_FILE}"
+
+        DONOR_BINLOGNAME=""
+        DONOR_MYSQL_VERSION=""
+
+    else
+        wsrep_log_error "******************* FATAL ERROR ********************** "
+        wsrep_log_error "Did not receive expected file from donor: '${SST_INFO_FILE}' or '${XB_GTID_INFO_FILE}'"
+        wsrep_log_error "****************************************************** "
+        exit 32
+    fi
+
+    if [[ -n "$DONOR_MYSQL_VERSION" ]]; then
+        local_version_str=""
+        donor_version_str=""
+
+        # Truncate the version numbers (we want the major.minor version like "5.6", not "5.6.35-...")
+        local_version_str=$(expr match "$MYSQL_VERSION" '\([0-9]\+\.[0-9]\+\)')
+        donor_version_str=$(expr match "$DONOR_MYSQL_VERSION" '\([0-9]\+\.[0-9]\+\)')
+
+        # Is this node's pxc version < donor's pxc version?
+        if ! check_for_version $local_version_str $donor_version_str; then
+            wsrep_log_error "******************* FATAL ERROR ********************** "
+            wsrep_log_error "FATAL: PXC is receiving an SST from a node with a higher version."
+            wsrep_log_error "This node's PXC version is $local_version_str.  The donor's PXC version is $donor_version_str."
+            wsrep_log_error "Upgrade this node before joining the cluster."
+            wsrep_log_error "****************************************************** "
+            exit 2
+        fi
+
+        # Is the donor's pxc version < this node's pxc version?
+        if ! check_for_version $donor_version_str $local_version_str; then
+            wsrep_log_warning "WARNING: PXC is receiving an SST from a node with a lower version."
+            wsrep_log_warning "This node's PXC version is $local_version_str. The donor's PXC version is $donor_version_str."
+            wsrep_log_warning "Run mysql_upgrade in non-cluster (standalone mode) to upgrade."
+            wsrep_log_warning "Check the upgrade process here:"
+            wsrep_log_warning "    https://www.percona.com/doc/percona-xtradb-cluster/LATEST/howtos/upgrade_guide.html"
+        fi
+    fi
 
     # server-id is already part of backup-my.cnf so avoid appending it.
     # server-id is the id of the node that is acting as donor and not joiner node.
@@ -1474,6 +1652,17 @@ then
 
     if [ ! -r "${STATDIR}/${IST_FILE}" ]
     then
+        get_stream
+        if [[ $encrypt -eq 1 && -n "$ecmd" ]]; then
+            strmcmd=" \$ecmd | $strmcmd"
+        fi
+        if [[ -n $sdecomp ]]; then
+            strmcmd=" $sdecomp | $strmcmd"
+        fi
+        if [[ $encrypt -eq 1 ]]; then
+            xbstreameopts=$xbstreameopts_sst
+        fi
+
         if [[ -d ${DATA}/.sst ]]; then
             wsrep_log_info "WARNING: Stale temporary SST directory: ${DATA}/.sst from previous state transfer. Removing"
             rm -rf ${DATA}/.sst
@@ -1488,15 +1677,17 @@ then
         # with ever increasing number of files and achieve nothing.
         find $ib_home_dir $ib_log_dir $ib_undo_dir $DATA -mindepth 1  -regex $cpat  -prune  -o -exec rm -rfv {} 1>/dev/null \+
 
+        # Clean the binlog dir (if it's explicitly specified)
+        # By default it'll be in the datadir
         tempdir=$(parse_cnf mysqld log-bin "")
-        if [[ -n ${tempdir:-} ]]; then
-            binlog_dir=$(dirname $tempdir)
-            binlog_file=$(basename $tempdir)
-            if [[ -n ${binlog_dir:-} && $binlog_dir != '.' && $binlog_dir != $DATA ]]; then
+        if [[ -n "$tempdir" ]]; then
+            binlog_dir=$(dirname "$tempdir")
+            binlog_file=$(basename "$tempdir")
+            if [[ -n ${binlog_dir:-} && "$binlog_dir" != '.' && "$binlog_dir" != "$DATA" ]];then
                 pattern="$binlog_dir/$binlog_file\.[0-9]+$"
                 wsrep_log_debug "Cleaning the binlog directory $binlog_dir as well"
-                find $binlog_dir -maxdepth 1 -type f -regex $pattern -exec rm -fv {} 1>&2 \+ || true
-                rm $binlog_dir/*.index || true
+                find "$binlog_dir" -maxdepth 1 -type f -regex $pattern -exec rm -fv {} 1>&2 \+ || true
+                rm -f $binlog_dir/*.index || true
             fi
         fi
 
@@ -1540,7 +1731,7 @@ then
             fi
 
             if [[ -n $progress ]] && pv --help | grep -q 'line-mode'; then
-                count=$(find ${DATA} -type f -name '*.qp' | wc -l)
+                count=$(find "${DATA}" -type f -name '*.qp' | wc -l)
                 count=$(( count*2 ))
                 if pv --help | grep -q FORMAT; then
                     pvopts="-f -s $count -l -N Decompression -F '%N => Rate:%r Elapsed:%t %e Progress: [%b/$count]'"
@@ -1562,7 +1753,7 @@ then
 
             if [[ $extcode -eq 0 ]]; then
                 wsrep_log_debug "Removing qpress files after decompression"
-                find ${DATA} -type f -name '*.qp' -delete
+                find "${DATA}" -type f -name '*.qp' -delete
                 if [[ $? -ne 0 ]]; then
                     wsrep_log_error "******************* FATAL ERROR ********************** "
                     wsrep_log_error "Something went wrong with deletion of qpress files. Investigate"
@@ -1576,17 +1767,30 @@ then
             fi
         fi
 
-        if  [[ ! -z $WSREP_SST_OPT_BINLOG ]]; then
+        if  [[ -n "$WSREP_SST_OPT_BINLOG" && -n "$DONOR_BINLOGNAME" ]]; then
 
-            BINLOG_DIRNAME=$(dirname $WSREP_SST_OPT_BINLOG)
-            BINLOG_FILENAME=$(basename $WSREP_SST_OPT_BINLOG)
+            binlog_dir=$(dirname "$WSREP_SST_OPT_BINLOG")
+            binlog_file=$(basename "$WSREP_SST_OPT_BINLOG")
+            donor_binlog_file=$DONOR_BINLOGNAME
 
-            # To avoid comparing data directory and BINLOG_DIRNAME
-            mv $DATA/${BINLOG_FILENAME}.* $BINLOG_DIRNAME/ 2>/dev/null || true
+            # rename the donor binlog to the name of the binlogs on the joiner
+            if [[ "$binlog_file" != "$donor_binlog_file" ]]; then
+                pushd "$DATA" &>/dev/null
+                for f in $donor_binlog_file.*; do
+                    if [[ ! -e "$f" ]]; then continue; fi
+                    f_new=$(echo $f | sed "s/$donor_binlog_file/$binlog_file/")
+                    mv "$f" "$f_new" 2>/dev/null || true
+                done
+                popd &> /dev/null
+            fi
 
-            pushd $BINLOG_DIRNAME &>/dev/null
-            for bfiles in $(ls -1 ${BINLOG_FILENAME}.* 2>/dev/null);do
-                echo ${BINLOG_DIRNAME}/${bfiles} >> ${BINLOG_FILENAME}.index
+            # To avoid comparing data directory and BINLOG_DIRNAME 
+            mv $DATA/${binlog_file}.* "$binlog_dir"/ 2>/dev/null || true
+
+            pushd "$binlog_dir" &>/dev/null
+            for bfiles in $binlog_file.*; do
+                if [[ ! -e "$bfiles" ]]; then continue; fi
+                echo "${binlog_dir}/${bfiles}" >> "${binlog_file}.index"
             done
             popd &> /dev/null
 
@@ -1601,7 +1805,7 @@ then
             wsrep_log_error "${INNOBACKUPEX_BIN} apply finished with errors." \
                             "Check ${DATA}/innobackup.prepare.log"
             echo "--------------- innobackup.prepare.log (START) --------------------" >&2
-            cat ${DATA}/innobackup.prepare.log >&2
+            cat "${DATA}/innobackup.prepare.log" >&2
             echo "--------------- innobackup.prepare.log (END) --------------------" >&2
             wsrep_log_error "****************************************************** "
             exit 22
@@ -1609,7 +1813,7 @@ then
 
         XB_GTID_INFO_FILE_PATH="${TDATA}/${XB_GTID_INFO_FILE}"
         set +e
-        rm $TDATA/innobackup.prepare.log $TDATA/innobackup.move.log 2> /dev/null
+        rm "$TDATA/innobackup.prepare.log" "$TDATA/innobackup.move.log" 2> /dev/null
         set -e
         wsrep_log_info "Moving the backup to ${TDATA}"
         timeit "Xtrabackup move stage" "$INNOMOVE"
@@ -1618,12 +1822,12 @@ then
             # Did we receive a keyring file?
             if [[ -r "${XB_DONOR_KEYRING_FILE_PATH}" ]]; then
                 wsrep_log_info "Moving sst keyring into place: moving $XB_DONOR_KEYRING_FILE_PATH to $keyring"
-                mv "${XB_DONOR_KEYRING_FILE_PATH}" $keyring
+                mv "${XB_DONOR_KEYRING_FILE_PATH}" "$keyring"
                 wsrep_log_debug "Keyring move successful"
             fi
 
             wsrep_log_debug "Move successful, removing ${DATA}"
-            rm -rf $DATA
+            rm -rf "$DATA"
             DATA=${TDATA}
         else
             if [[ -r "${XB_DONOR_KEYRING_FILE_PATH}" ]]; then
@@ -1633,7 +1837,7 @@ then
             wsrep_log_error "Move failed, keeping ${DATA} for further diagnosis" \
                             "Check ${DATA}/innobackup.move.log for details"
             echo "--------------- innobackup.move.log (START) --------------------" >&2
-            cat ${DATA}/innobackup.move.log >&2
+            cat "${DATA}/innobackup.move.log" >&2
             echo "--------------- innobackup.move.log (END) --------------------" >&2
             wsrep_log_error "****************************************************** "
             exit 22
@@ -1649,7 +1853,7 @@ then
         wsrep_log_error "****************************************************** "
         exit 2
     fi
-    wsrep_log_info "Galera co-ords from recovery: $(cat ${XB_GTID_INFO_FILE_PATH})"
+    wsrep_log_info "Galera co-ords from recovery: $(cat "${XB_GTID_INFO_FILE_PATH}")"
     cat "${XB_GTID_INFO_FILE_PATH}" # output UUID:seqno
     wsrep_log_debug "Total time on joiner: $totime seconds"
 fi
