@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of the License.
@@ -28,6 +28,12 @@
 #include "prealloced_array.h"
 #include "tztime.h"
 #include "crypt_genhash_impl.h"         /* CRYPT_MAX_PASSWORD_SIZE */
+#include "sql_user_table.h"
+
+#ifndef DBUG_OFF
+#define HASH_STRING_WITH_QUOTE \
+        "$5$BVZy9O>'a+2MH]_?$fpWyabcdiHjfCVqId/quykZzjaA7adpkcen/uiQrtmOK4p4"
+#endif
 
 /**
   Auxiliary function for constructing a  user list string.
@@ -180,6 +186,32 @@ void append_user_new(THD *thd, String *str, LEX_USER *user, bool comma= true)
   }
 }
 
+/**
+  Escapes special characters in the unescaped string, taking into account
+  the current character set and sql mode.
+
+  @param thd    [in]  The thd structure.
+  @param to     [out] Escaped string output buffer.
+  @param from   [in]  String to escape.
+  @param length [in]  String to escape length.
+
+  @return Result value.
+    @retval != (ulong)-1 Succeeded. Number of bytes written to the output
+                         buffer without the '\0' character.
+    @retval (ulong)-1    Failed.
+*/
+
+inline ulong escape_string_mysql(THD *thd, char *to, const char *from,
+                                 ulong length)
+{
+    if (!(thd->variables.sql_mode & MODE_NO_BACKSLASH_ESCAPES))
+      return (uint)escape_string_for_mysql(system_charset_info, to, 0, from,
+                                           length);
+    else
+      return (uint)escape_quotes_for_mysql(system_charset_info, to, 0, from,
+                                           length, '\'');
+}
+
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
 
 /*
@@ -434,7 +466,8 @@ bool set_and_validate_user_attributes(THD *thd,
         always check for password expire/interval attributes as there is no
         way to differentiate NEVER EXPIRE and EXPIRE DEFAULT scenario
       */
-      what_to_set|= PASSWORD_EXPIRE_ATTR;
+      if (Str->alter_status.update_password_expired_fields)
+        what_to_set|= PASSWORD_EXPIRE_ATTR;
     }
     else
     {
@@ -620,15 +653,18 @@ bool change_password(THD *thd, const char *host, const char *user,
 {
   TABLE_LIST tables;
   TABLE *table;
+  Acl_table_intact table_intact;
   LEX_USER *combo= NULL;
   /* Buffer should be extended when password length is extended. */
-  char buff[512];
+  char buff[2048];
   /* buffer to store the hash string */
   char hash_str[MAX_FIELD_WIDTH]= {0};
+  char *hash_str_escaped= NULL;
   ulong query_length= 0;
   ulong what_to_set= 0;
   bool save_binlog_row_based;
   size_t new_password_len= strlen(new_password);
+  size_t escaped_hash_str_len= 0;
   bool result= true, rollback_whole_statement= false;
   int ret;
 
@@ -654,7 +690,6 @@ bool change_password(THD *thd, const char *host, const char *user,
   }
 #endif /* WITH_WSREP */
 
-
   tables.init_one_table("mysql", 5, "user", 4, "user", TL_WRITE);
 
 #ifdef HAVE_REPLICATION
@@ -677,6 +712,9 @@ bool change_password(THD *thd, const char *host, const char *user,
   if (!(table= open_ltable(thd, &tables, TL_WRITE, MYSQL_LOCK_IGNORE_TIMEOUT)))
     DBUG_RETURN(1);
 
+  if (table_intact.check(table, &mysql_user_table_def))
+    DBUG_RETURN(1);
+
   if (!table->key_info)
   {
     my_error(ER_TABLE_CORRUPT, MYF(0), table->s->db.str,
@@ -756,6 +794,7 @@ bool change_password(THD *thd, const char *host, const char *user,
     mysql_mutex_unlock(&acl_cache->lock);
     goto end;
   }
+
   ret= replace_user_table(thd, table, combo, 0, false, true, what_to_set);
   if (ret)
   {
@@ -777,6 +816,20 @@ bool change_password(THD *thd, const char *host, const char *user,
 
   mysql_mutex_unlock(&acl_cache->lock);
   result= 0;
+  escaped_hash_str_len= (opt_log_builtin_as_identified_by_password?
+                         combo->auth.length:
+                         acl_user->auth_string.length)*2+1;
+  /*
+     Allocate a buffer for the escaped password. It should at least have place
+     for length*2+1 chars.
+  */
+  hash_str_escaped= (char *)alloc_root(thd->mem_root, escaped_hash_str_len);
+  if (!hash_str_escaped)
+  {
+    my_error(ER_OUTOFMEMORY, MYF(ME_FATALERROR), 0);
+    result= 1;
+    goto end;
+  }
   /*
     Based on @@log-backward-compatible-user-definitions variable
     rewrite SET PASSWORD
@@ -784,17 +837,33 @@ bool change_password(THD *thd, const char *host, const char *user,
   if (opt_log_builtin_as_identified_by_password)
   {
     memcpy(hash_str, combo->auth.str, combo->auth.length);
+
+    DBUG_EXECUTE_IF("force_hash_string_with_quote",
+		     strcpy(hash_str, HASH_STRING_WITH_QUOTE);
+                   );
+
+    escape_string_mysql(thd, hash_str_escaped, hash_str, strlen(hash_str));
+
     query_length= sprintf(buff, "SET PASSWORD FOR '%-.120s'@'%-.120s'='%s'",
                           acl_user->user ? acl_user->user : "",
                           acl_user->host.get_host() ? acl_user->host.get_host() : "",
-                          hash_str);
+                          hash_str_escaped);
   }
   else
+  {
+    DBUG_EXECUTE_IF("force_hash_string_with_quote",
+                     strcpy(acl_user->auth_string.str, HASH_STRING_WITH_QUOTE);
+                   );
+
+    escape_string_mysql(thd, hash_str_escaped, acl_user->auth_string.str,
+                        strlen(acl_user->auth_string.str));
+
     query_length= sprintf(buff, "ALTER USER '%-.120s'@'%-.120s' IDENTIFIED WITH '%-.120s' AS '%s'",
                           acl_user->user ? acl_user->user : "",
                           acl_user->host.get_host() ? acl_user->host.get_host() : "",
                           acl_user->plugin.str,
-                          acl_user->auth_string.str);
+                          hash_str_escaped);
+  }
   result= write_bin_log(thd, true, buff, query_length,
                         table->file->has_transactions());
 end:
@@ -824,7 +893,7 @@ end:
 
 #ifdef WITH_WSREP
 error:
-  WSREP_ERROR("Replication of SET PASSWORD failed: %s", buff);
+  WSREP_ERROR("Fail to replicate SET PASSWORD (%s)", buff);
   DBUG_RETURN(result);
 #endif /* WITH_WSREP */
 }
@@ -1119,6 +1188,7 @@ static int handle_grant_data(TABLE_LIST *tables, bool drop,
   int result= 0;
   int found;
   int ret;
+  Acl_table_intact table_intact;
   DBUG_ENTER("handle_grant_data");
 
   /* Handle special utility user */
@@ -1139,6 +1209,12 @@ static int handle_grant_data(TABLE_LIST *tables, bool drop,
   }
 
   /* Handle user table. */
+  if (table_intact.check(tables[0].table, &mysql_user_table_def))
+  {
+    result= -1;
+    goto end;
+  }
+
   if ((found= handle_grant_table(tables, 0, drop, user_from, user_to)) < 0)
   {
     /* Handle of table failed, don't touch the in-memory array. */
@@ -1163,6 +1239,12 @@ static int handle_grant_data(TABLE_LIST *tables, bool drop,
   }
 
   /* Handle db table. */
+  if (table_intact.check(tables[1].table, &mysql_db_table_def))
+  {
+    result= -1;
+    goto end;
+  }
+
   if ((found= handle_grant_table(tables, 1, drop, user_from, user_to)) < 0)
   {
     /* Handle of table failed, don't touch the in-memory array. */
@@ -1187,6 +1269,12 @@ static int handle_grant_data(TABLE_LIST *tables, bool drop,
   }
 
   /* Handle stored routines table. */
+  if (table_intact.check(tables[4].table, &mysql_procs_priv_table_def))
+  {
+    result= -1;
+    goto end;
+  }
+
   if ((found= handle_grant_table(tables, 4, drop, user_from, user_to)) < 0)
   {
     /* Handle of table failed, don't touch in-memory array. */
@@ -1227,6 +1315,12 @@ static int handle_grant_data(TABLE_LIST *tables, bool drop,
   }
 
   /* Handle tables table. */
+  if (table_intact.check(tables[2].table, &mysql_tables_priv_table_def))
+  {
+    result= -1;
+    goto end;
+  }
+
   if ((found= handle_grant_table(tables, 2, drop, user_from, user_to)) < 0)
   {
     /* Handle of table failed, don't touch columns and in-memory array. */
@@ -1243,6 +1337,12 @@ static int handle_grant_data(TABLE_LIST *tables, bool drop,
     }
 
     /* Handle columns table. */
+    if (table_intact.check(tables[3].table, &mysql_columns_priv_table_def))
+    {
+      result= -1;
+      goto end;
+    }
+
     if ((found= handle_grant_table(tables, 3, drop, user_from, user_to)) < 0)
     {
       /* Handle of table failed, don't touch the in-memory array. */
@@ -1263,6 +1363,12 @@ static int handle_grant_data(TABLE_LIST *tables, bool drop,
   /* Handle proxies_priv table. */
   if (tables[5].table)
   {
+    if (table_intact.check(tables[5].table, &mysql_proxies_priv_table_def))
+    {
+      result= -1;
+      goto end;
+    }
+
     if ((found= handle_grant_table(tables, 5, drop, user_from, user_to)) < 0)
     {
       /* Handle of table failed, don't touch the in-memory array. */
@@ -1425,14 +1531,21 @@ bool mysql_create_user(THD *thd, List <LEX_USER> &list, bool if_not_exists)
     rlb->mem_free();
     mysql_rewrite_create_alter_user(thd, rlb, &users_not_to_log);
 
-    if (!thd->rewritten_query.length())
-      result|= write_bin_log(thd, false, thd->query().str, thd->query().length,
-                             transactional_tables);
-    else
-      result|= write_bin_log(thd, false,
-                             thd->rewritten_query.c_ptr_safe(),
-                             thd->rewritten_query.length(),
-                             transactional_tables);
+    int ret= commit_owned_gtid_by_partial_command(thd);
+
+    if (ret == 1)
+    {
+      if (!thd->rewritten_query.length())
+        result|= write_bin_log(thd, false, thd->query().str, thd->query().length,
+                               transactional_tables);
+      else
+        result|= write_bin_log(thd, false,
+                               thd->rewritten_query.c_ptr_safe(),
+                               thd->rewritten_query.length(),
+                               transactional_tables);
+    }
+    else if (ret == -1)
+      result|= -1;
   }
 
   lock.unlock();
@@ -1692,8 +1805,14 @@ bool mysql_rename_user(THD *thd, List <LEX_USER> &list)
     my_error(ER_CANNOT_USER, MYF(0), "RENAME USER", wrong_users.c_ptr_safe());
   
   if (some_users_renamed)
-    result |= write_bin_log(thd, FALSE, thd->query().str, thd->query().length,
-                            transactional_tables);
+  {
+    int ret= commit_owned_gtid_by_partial_command(thd);
+    if (ret == 1)
+      result|= write_bin_log(thd, FALSE, thd->query().str, thd->query().length,
+                              transactional_tables);
+    else if (ret == -1)
+      result|= -1;
+  }
 
   lock.unlock();
 
@@ -1740,6 +1859,7 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
   bool is_privileged_user= false;
   bool rollback_whole_statement= false;
   std::set<LEX_USER *> users_not_to_log;
+  Acl_table_intact table_intact;
 
   DBUG_ENTER("mysql_alter_user");
 
@@ -1770,6 +1890,9 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
   if (!(table= open_ltable(thd, &tables, TL_WRITE, MYSQL_LOCK_IGNORE_TIMEOUT)))
     DBUG_RETURN(true);
 
+  if (table_intact.check(table, &mysql_user_table_def))
+    DBUG_RETURN(true);
+
   if (!table->key_info)
   {
     my_error(ER_TABLE_CORRUPT, MYF(0), table->s->db.str,
@@ -1907,10 +2030,14 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
     rlb->mem_free();
     mysql_rewrite_create_alter_user(thd, rlb, &users_not_to_log);
 
-    result|= (write_bin_log(thd, false,
-                            thd->rewritten_query.c_ptr_safe(),
-                            thd->rewritten_query.length(),
-                            table->file->has_transactions()) != 0);
+    int ret= commit_owned_gtid_by_partial_command(thd);
+    if (ret == 1)
+      result|= (write_bin_log(thd, false,
+                              thd->rewritten_query.c_ptr_safe(),
+                              thd->rewritten_query.length(),
+                              table->file->has_transactions()) != 0);
+    else if (ret == -1)
+      result|= -1;
   }
 
   lock.unlock();
