@@ -1,4 +1,4 @@
-/* Copyright (c) 2005, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2005, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -1312,34 +1312,6 @@ Event_job_data::construct_sp_sql(THD *thd, String *sp_sql)
 }
 
 
-/**
-  Get DROP EVENT statement to binlog the drop of ON COMPLETION NOT
-  PRESERVE event.
-*/
-
-bool
-Event_job_data::construct_drop_event_sql(THD *thd, String *sp_sql)
-{
-  LEX_STRING buffer;
-  const uint STATIC_SQL_LENGTH= 14;
-
-  DBUG_ENTER("Event_job_data::construct_drop_event_sql");
-
-  buffer.length= STATIC_SQL_LENGTH + name.length*2 + dbname.length*2;
-  if (! (buffer.str= (char*) thd->alloc(buffer.length)))
-    DBUG_RETURN(TRUE);
-
-  sp_sql->set(buffer.str, buffer.length, system_charset_info);
-  sp_sql->length(0);
-
-  sp_sql->append(C_STRING_WITH_LEN("DROP EVENT "));
-  append_identifier(thd, sp_sql, dbname.str, dbname.length);
-  sp_sql->append('.');
-  append_identifier(thd, sp_sql, name.str, name.length);
-
-  DBUG_RETURN(thd->is_fatal_error);
-}
-
 /**
   Compiles and executes the event (the underlying sp_head object)
 
@@ -1489,7 +1461,7 @@ end:
       Construct a query for the binary log, to ensure the event is dropped
       on the slave
     */
-    if (construct_drop_event_sql(thd, &sp_sql))
+    if (construct_drop_event_sql(thd, &sp_sql, dbname, name))
       ret= 1;
     else
     {
@@ -1519,6 +1491,7 @@ end:
           // sizeof(LEX) = 4512, so it's relatively safe to allocate it on stack.
           LEX lex;
           lex.sql_command = SQLCOM_DROP_EVENT;
+          lex.select_lex = NULL;
           thd->lex = &lex;
           WSREP_TO_ISOLATION_BEGIN(WSREP_MYSQL_DB, NULL, NULL);
           thd->lex= original_saved_lex;
@@ -1552,6 +1525,37 @@ error:
   DBUG_RETURN(ret);
 }
 
+/**
+  Get DROP EVENT statement to binlog the drop of ON COMPLETION NOT
+  PRESERVE event.
+*/
+bool construct_drop_event_sql(THD *thd, String *sp_sql,
+                              const LEX_STRING &schema_name,
+                              const LEX_STRING &event_name)
+{
+  LEX_STRING buffer;
+  const uint STATIC_SQL_LENGTH= 14;
+  int ret= 0;
+
+  DBUG_ENTER("construct_drop_event_sql");
+
+  buffer.length= STATIC_SQL_LENGTH + event_name.length*2 +
+                 schema_name.length * 2;
+  if (! (buffer.str= (char*) thd->alloc(buffer.length)))
+    DBUG_RETURN(true);
+
+  sp_sql->set(buffer.str, buffer.length, system_charset_info);
+  sp_sql->length(0);
+
+  ret|= sp_sql->append(C_STRING_WITH_LEN("DROP EVENT IF EXISTS"));
+  append_identifier(thd, sp_sql, schema_name.str, schema_name.length);
+  ret|= sp_sql->append('.');
+  append_identifier(thd, sp_sql, event_name.str,
+                    event_name.length);
+
+  DBUG_RETURN(ret);
+}
+
 
 /*
   Checks whether two events are in the same schema
