@@ -381,7 +381,7 @@ void Proc_table_intact::report_error(uint code, const char *fmt, ...)
   my_vsnprintf(buf, sizeof(buf), fmt, args);
   va_end(args);
 
-  if (code)
+  if (code == ER_COL_COUNT_DOESNT_MATCH_CORRUPTED_V2)
     my_message(code, buf, MYF(0));
   else
     my_error(ER_CANNOT_LOAD_FROM_TABLE_V2, MYF(0), "mysql", "proc");
@@ -2572,6 +2572,7 @@ uint sp_get_flags_for_command(LEX *lex)
     flags= sp_head::HAS_SQLCOM_RESET;
     break;
   case SQLCOM_CREATE_INDEX:
+  case SQLCOM_CREATE_COMPRESSION_DICTIONARY:
   case SQLCOM_CREATE_DB:
   case SQLCOM_CREATE_VIEW:
   case SQLCOM_CREATE_TRIGGER:
@@ -2583,6 +2584,7 @@ uint sp_get_flags_for_command(LEX *lex)
   case SQLCOM_RENAME_TABLE:
   case SQLCOM_RENAME_USER:
   case SQLCOM_DROP_INDEX:
+  case SQLCOM_DROP_COMPRESSION_DICTIONARY:
   case SQLCOM_DROP_DB:
   case SQLCOM_REVOKE_ALL:
   case SQLCOM_DROP_USER:
@@ -2870,8 +2872,8 @@ int wsrep_create_sp(THD *thd, uchar** buf, size_t* buf_len)
                      thd->lex->definer->host,
                      saved_mode))
   {
-    WSREP_WARN("SP create string failed: schema: %s, query: %s",
-               (thd->db().length ? thd->db().str : "(null)"), thd->query().str);
+    WSREP_WARN("Store Procedure create string failed: schema: %s, query: %s",
+               (thd->db().str ? thd->db().str : "(null)"),thd->query().str);
     return 1;
   }
   return wsrep_to_buf_helper(thd, log_query.ptr(), log_query.length(), buf, buf_len);
