@@ -427,16 +427,24 @@ inline bool too_many_busy_threads(thread_group_t *thread_group)
 /*
    Checks if a given connection is eligible to enter the high priority queue
    based on its current thread_pool_high_prio_mode value, available high
-   priority tickets and transactional state.
+   priority tickets and transactional state and whether any locks are held.
 */
 
-inline bool connection_is_high_prio(connection_t *c)
+inline bool connection_is_high_prio(const connection_t *c)
 {
   const ulong mode= c->thd->variables.threadpool_high_prio_mode;
 
   return (mode == TP_HIGH_PRIO_MODE_STATEMENTS) ||
-    (mode == TP_HIGH_PRIO_MODE_TRANSACTIONS &&
-     c->tickets > 0 && thd_is_transaction_active(c->thd));
+    (mode == TP_HIGH_PRIO_MODE_TRANSACTIONS && c->tickets > 0 &&
+     (thd_is_transaction_active(c->thd) ||
+      c->thd->variables.option_bits & OPTION_TABLE_LOCK ||
+      c->thd->locked_tables_mode != LTM_NONE ||
+      c->thd->mdl_context.has_locks() ||
+      c->thd->global_read_lock.is_acquired() ||
+      c->thd->backup_tables_lock.is_acquired() ||
+      c->thd->backup_binlog_lock.is_acquired() ||
+      c->thd->mdl_context.has_locks(MDL_key::USER_LEVEL_LOCK) ||
+      c->thd->mdl_context.has_locks(MDL_key::LOCKING_SERVICE)));
 }
 
 } // namespace
@@ -1293,6 +1301,7 @@ static void wait_end(thread_group_t *thread_group)
 static connection_t *alloc_connection(THD *thd)
 {
   DBUG_ENTER("alloc_connection");
+  DBUG_EXECUTE_IF("simulate_tp_alloc_connection_oom", DBUG_RETURN(NULL););
 
   connection_t* connection = (connection_t *)
       my_malloc(key_memory_thread_pool_connection,
@@ -1320,6 +1329,23 @@ bool Thread_pool_connection_handler::add_connection(Channel_info *channel_info)
   DBUG_ENTER("Thread_pool_connection_handler::add_connection");
 
   THD* thd= channel_info->create_thd();
+
+  if (unlikely(!thd))
+  {
+    channel_info->send_error_and_close_channel(ER_OUT_OF_RESOURCES, 0, false);
+    DBUG_RETURN(true);
+  }
+
+  connection_t *connection= alloc_connection(thd);
+
+  if (unlikely(!connection))
+  {
+    thd->get_protocol_classic()->end_net();
+    delete thd;
+    channel_info->send_error_and_close_channel(ER_OUT_OF_RESOURCES, 0, false);
+    DBUG_RETURN(true);
+  }
+
   delete channel_info;
 
   thd->set_new_thread_id();
@@ -1329,15 +1355,6 @@ bool Thread_pool_connection_handler::add_connection(Channel_info *channel_info)
 
   Global_THD_manager::get_instance()->add_thd(thd);
 
-  connection_t *connection= alloc_connection(thd);
-
-  if (!connection)
-  {
-    /* Allocation failed */
-    threadpool_remove_connection(thd);
-    DBUG_RETURN(true);
-  }
-
   thd->event_scheduler.data= connection;
 
   /* Assign connection to a group. */
@@ -1463,7 +1480,7 @@ static void set_wait_timeout(connection_t *c)
 
   c->abs_wait_timeout= pool_timer.current_microtime +
     1000LL*pool_timer.tick_interval +
-    1000000LL*c->thd->variables.net_wait_timeout;
+    1000000LL*c->thd->get_wait_timeout();
 
   set_next_timeout_check(c->abs_wait_timeout);
   DBUG_VOID_RETURN;
@@ -1768,26 +1785,29 @@ A likely cause of pool blocks are clients that lock resources for long time. \
 
 static void print_pool_blocked_message(bool max_threads_reached)
 {
-  ulonglong now;
+  ulonglong now= my_microsecond_getsystime();
   static bool msg_written;
-  
-  now= my_microsecond_getsystime();
+
   if (pool_block_start == 0)
   {
     pool_block_start= now;
-    msg_written = false;
-    return;
+    msg_written= false;
   }
-  
-  if (now > pool_block_start + BLOCK_MSG_DELAY && !msg_written)
+
+  if (!msg_written
+      && ((now > pool_block_start + BLOCK_MSG_DELAY)
+          || (now == pool_block_start)))
   {
     if (max_threads_reached)
       sql_print_error(MAX_THREADS_REACHED_MSG);
     else
       sql_print_error(CREATE_THREAD_ERROR_MSG, my_errno());
-    
-    sql_print_information("Threadpool has been blocked for %u seconds\n",
-      (uint)((now- pool_block_start)/1000000));
+
+    if (now > pool_block_start)
+    {
+      sql_print_information("Threadpool has been blocked for %u seconds\n",
+                            (uint)((now - pool_block_start)/1000000));
+    }
     /* avoid reperated messages for the same blocking situation */
     msg_written= true;
   }
