@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -178,7 +178,9 @@ bool trans_begin(THD *thd, uint flags)
 
   DBUG_EXECUTE_IF("dbug_set_high_prio_trx", {
     DBUG_ASSERT(thd->tx_priority==0);
+#ifdef WITH_WSREP
     WSREP_WARN("InnoDB High Priority being used: %d -> %d", thd->tx_priority, 1);
+#endif /* WITH_WSREP */
     thd->tx_priority= 1;
   });
 
@@ -672,15 +674,6 @@ bool trans_savepoint(THD *thd, LEX_STRING name)
       !opt_using_transactions)
     DBUG_RETURN(FALSE);
 
-  if (thd->variables.transaction_write_set_extraction != HASH_ALGORITHM_OFF)
-  {
-    // is_fatal_errror is needed to avoid stored procedures to skip the error.
-    thd->is_fatal_error= 1;
-    my_error(ER_OPTION_PREVENTS_STATEMENT, MYF(0),
-             "--transaction-write-set-extraction!=OFF");
-    DBUG_RETURN(true);
-  }
-
   if (thd->get_transaction()->xid_state()->check_has_uncommitted_xa())
     DBUG_RETURN(true);
 
@@ -724,6 +717,12 @@ bool trans_savepoint(THD *thd, LEX_STRING name)
   */
   newsv->mdl_savepoint= thd->mdl_context.mdl_savepoint();
 
+  if (thd->is_current_stmt_binlog_row_enabled_with_write_set_extraction())
+  {
+    thd->get_transaction()->get_transaction_write_set_ctx()
+        ->add_savepoint(name.str);
+  }
+
   DBUG_RETURN(FALSE);
 }
 
@@ -812,6 +811,12 @@ bool trans_rollback_to_savepoint(THD *thd, LEX_STRING name)
   if (!res && mdl_can_safely_rollback_to_savepoint)
     thd->mdl_context.rollback_to_savepoint(sv->mdl_savepoint);
 
+  if (thd->is_current_stmt_binlog_row_enabled_with_write_set_extraction())
+  {
+    thd->get_transaction()->get_transaction_write_set_ctx()
+        ->rollback_to_savepoint(name.str);
+  }
+
   DBUG_RETURN(MY_TEST(res));
 }
 
@@ -850,5 +855,11 @@ bool trans_release_savepoint(THD *thd, LEX_STRING name)
 
   thd->get_transaction()->m_savepoints= sv->prev;
 
+  if (thd->is_current_stmt_binlog_row_enabled_with_write_set_extraction())
+  {
+    thd->get_transaction()->get_transaction_write_set_ctx()
+        ->del_savepoint(name.str);
+  }
+
   DBUG_RETURN(MY_TEST(res));
 }
