@@ -49,6 +49,10 @@ extern my_bool wsrep_debug;
 #endif /* WITH_WSREP */
 #include "row0mysql.h"
 #include "dict0dict.h"
+#ifdef WITH_WSREP
+extern "C" int wsrep_trx_is_aborting(void *thd_ptr);
+extern my_bool wsrep_debug;
+#endif
 
 /** Number of times a thread is allowed to enter InnoDB within the same
 SQL query after it has once got the ticket. */
@@ -318,6 +322,13 @@ srv_conc_force_exit_innodb(
 
 		return;
 	}
+#ifdef WITH_WSREP
+	if (wsrep_on(trx->mysql_thd) && 
+	    wsrep_trx_is_aborting(trx->mysql_thd)) {
+		srv_conc_force_enter_innodb(trx);
+		return;
+	}
+#endif /* WITH_WSREP */
 
 	srv_conc_exit_innodb_with_atomics(trx);
 
