@@ -23,6 +23,8 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 
 #ident "Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved."
 
+#if TOKU_INCLUDE_UPSERT
+
 // Point updates and upserts
 
 // Restrictions:
@@ -128,7 +130,7 @@ static void dump_item_list(const char* h, List<Item> &l) {
 }
 
 // Find a Field by its Item name
-static Field* find_field_by_name(TOKUDB_UNUSED(TABLE* table), Item* item) {
+static Field* find_field_by_name(TABLE* table, Item* item) {
     if (item->type() != Item::FIELD_ITEM)
         return NULL;
     Item_field* field_item = static_cast<Item_field*>(item);
@@ -189,9 +191,12 @@ static uint32_t var_field_index(
     return v_index;    
 }
 
-static uint32_t blob_field_index(TABLE* table,
-                                 KEY_AND_COL_INFO* kc_info,
-                                 uint field_num) {
+static uint32_t blob_field_index(
+    TABLE* table,
+    KEY_AND_COL_INFO* kc_info,
+    uint idx,
+    uint field_num) {
+
     assert_always(field_num < table->s->fields);
     uint b_index;
     for (b_index = 0; b_index < kc_info->num_blobs; b_index++) {
@@ -216,46 +221,45 @@ int ha_tokudb::fast_update(
     TOKUDB_HANDLER_DBUG_ENTER("");
     int error = 0;
 
-    if (!tokudb::sysvars::enable_fast_update(thd)) {
-        error = ENOTSUP;
-        goto exit;
-    }
-
     if (TOKUDB_UNLIKELY(TOKUDB_DEBUG_FLAGS(TOKUDB_DEBUG_UPSERT))) {
         dump_item_list("fields", update_fields);
         dump_item_list("values", update_values);
         if (conds) {
-            fprintf(stderr, "conds\n");
-            dump_item(conds);
-            fprintf(stderr, "\n");
+            fprintf(stderr, "conds\n"); dump_item(conds); fprintf(stderr, "\n");
         }
     }
 
     if (update_fields.elements < 1 ||
         update_fields.elements != update_values.elements) {
         error = ENOTSUP;  // something is fishy with the parameters
-        goto exit;
+        goto return_error;
     }
-
+    
     if (!check_fast_update(thd, update_fields, update_values, conds)) {
-        error = HA_ERR_UNSUPPORTED;
-        goto exit;
+        error = ENOTSUP;
+        goto check_error;
     }
 
     error = send_update_message(
-        update_fields, update_values, conds, transaction);
-
-    if (error) {
-        int mapped_error = map_to_handler_error(error);
-        if (mapped_error == error)
-            error = HA_ERR_UNSUPPORTED;
+        update_fields,
+        update_values,
+        conds,
+        transaction);
+    if (error != 0) {
+        goto check_error;
     }
 
-exit:
-
-    if (error != 0 && error != ENOTSUP)
-        print_error(error, MYF(0));
+check_error:
+    if (error != 0) {
+#if TOKU_INCLUDE_UPSERT
+        if (tokudb::sysvars::disable_slow_update(thd) != 0)
+            error = HA_ERR_UNSUPPORTED;
+#endif
+        if (error != ENOTSUP)
+            print_error(error, MYF(0));
+    }
 
+return_error:
     TOKUDB_HANDLER_DBUG_RETURN(error);
 }
 
@@ -771,7 +775,11 @@ static void marshall_update(
         update_operation = '=';
         field_type = lhs_field->binary() ? UPDATE_TYPE_BLOB : UPDATE_TYPE_TEXT;
         offset =
-            blob_field_index(table, &share->kc_info, lhs_field->field_index);
+            blob_field_index(
+                table,
+                &share->kc_info,
+                table->s->primary_key,
+                lhs_field->field_index);
         v_str = *rhs_item->val_str(&v_str);
         v_length = v_str.length();
         if (v_length >= lhs_field->max_data_length()) {
@@ -912,7 +920,7 @@ int ha_tokudb::send_update_message(
         marshall_update(update_message, lhs_item, rhs_item, table, share);
     }
 
-    rwlock_t_lock_read(share->_num_DBs_lock);
+    share->_num_DBs_lock.lock_read();
 
     // hot index in progress
     if (share->num_DBs > table->s->keys + tokudb_test(hidden_primary_key)) {
@@ -947,12 +955,8 @@ int ha_tokudb::upsert(
     List<Item>& update_values) {
 
     TOKUDB_HANDLER_DBUG_ENTER("");
-    int error = 0;
 
-    if (!tokudb::sysvars::enable_fast_upsert(thd)) {
-        error = ENOTSUP;
-        goto exit;
-    }
+    int error = 0;
 
     if (TOKUDB_UNLIKELY(TOKUDB_DEBUG_FLAGS(TOKUDB_DEBUG_UPSERT))) {
         fprintf(stderr, "upsert\n");
@@ -964,27 +968,30 @@ int ha_tokudb::upsert(
     if (update_fields.elements < 1 ||
         update_fields.elements != update_values.elements) {
         error = ENOTSUP;
-        goto exit;
+        goto return_error;
     }
 
     if (!check_upsert(thd, update_fields, update_values)) {
-        error = HA_ERR_UNSUPPORTED;
-        goto exit;
+        error = ENOTSUP;
+        goto check_error;
+    }
+    
+    error = send_upsert_message(thd, update_fields, update_values, transaction);
+    if (error != 0) {
+        goto check_error;
     }
 
-    error = send_upsert_message(update_fields, update_values, transaction);
-
-    if (error) {
-        int mapped_error = map_to_handler_error(error);
-        if (mapped_error == error)
+check_error:
+    if (error != 0) {
+#if TOKU_INCLUDE_UPSERT
+        if (tokudb::sysvars::disable_slow_upsert(thd) != 0)
             error = HA_ERR_UNSUPPORTED;
+#endif
+        if (error != ENOTSUP)
+            print_error(error, MYF(0));
     }
 
-exit:
-
-    if (error != 0 && error != ENOTSUP)
-        print_error(error, MYF(0));
-
+return_error:
     TOKUDB_HANDLER_DBUG_RETURN(error);
 }
 
@@ -1033,6 +1040,7 @@ bool ha_tokudb::check_upsert(
 // Generate an upsert message and send it into the primary tree.
 // Return 0 if successful.
 int ha_tokudb::send_upsert_message(
+    THD* thd,
     List<Item>& update_fields,
     List<Item>& update_values,
     DB_TXN* txn) {
@@ -1104,7 +1112,7 @@ int ha_tokudb::send_upsert_message(
         marshall_update(update_message, lhs_item, rhs_item, table, share);
     }
 
-    rwlock_t_lock_read(share->_num_DBs_lock);
+    share->_num_DBs_lock.lock_read();
 
     // hot index in progress
     if (share->num_DBs > table->s->keys + tokudb_test(hidden_primary_key)) {
@@ -1127,3 +1135,5 @@ int ha_tokudb::send_upsert_message(
 
     return error;
 }
+
+#endif
